n <- 20
x <- sort(runif(n, 0, 20))
epsilon <- rnorm(n, 0, 2.5)
y <- 2 + 0.5*x + epsilon

plot(x, y)

cat( "model {
      	for (i in 1:n){
		y[i] ~ dnorm(mu[i], tau)
		mu[i] <- b0 + b1 * x[i]
	}
	b0 ~ dnorm(0, .0001)
	b1 ~ dnorm(0, .0001)
	tau <- pow(sigma, -2)
	sigma ~ dunif(0, 100)
}", file="reglin.modelo")

## alternativa:
## tau ~ dgamma(0.001, 0.001)
## sigma2 <- 1/tau

require(rjags)
# require(R2jags)
# require(dclone)

inits <- list(list(b0=0, b1=1, sigma=1),
             list(b0=2, b1=0.1, sigma=5))

jags <- jags.model('reglin.modelo',
                   data = list('x' = x,
                               'y' = y,
                               'n' = n),
                   n.chains = 2,
                   inits=inits,
                   n.adapt = 100)

class(jags)

sam <- jags.samples(jags,
             c('b0', 'b1', 'sigma'),
             1000)
class(sam)

sam <- coda.samples(jags,
             c('b0', 'b1', 'sigma'),
             1000)
class(sam)
str(sam)
plot(sam)

sam <- coda.samples(jags,
             c('b0', 'b1', 'sigma', "y"),
             1000)
str(sam)

int <- HPDinterval(sam)
str(int)
int

##======================================================================
## BRMS/STAN

da <- data.frame(x = x, y = y)
m0b <- brm(y ~ x, data = da, cores = 4)

class(m0b)
m0b
summary(m0b)
posterior_summary(m0b)
posterior_interval(m0b)
plot(m0b)
pairs(m0b)
prior_summary(m0b)
waic(m0b)
loo(m0b)
vcov(m0b)
fixef(m0b)

##----------------------------------------------------------------------
m0b$prior

aa <- as_draws_df(m0b)
str(aa)
acf(aa$b_Intercept)
acf(aa$b_x)
acf(aa$sigma)
##----------------------------------------------------------------------

(h1 <- hypothesis(m0b, "x = 0"))
plot(h1)

mcmc_plot(m0b)
mcmc_plot(m0b, variable = c("b_Intercept", "b_x"))
mcmc_plot(m0b, variable = c("b_Intercept", "b_x"), type = "acf")
mcmc_plot(m0b, variable = "sigma")
mcmc_plot(m0b, variable = "sigma", type = "acf")
## show histograms of the posterior distributions
mcmc_plot(m0b, type = "hist")

stancode(m0b)

posterior_table(m0b)
## post_prob(m0b)
pp_check(m0b)
pi <- predictive_interval(m0b)
str(pi)
head(pi)
pe <- predictive_error(m0b)
hist(pe)

launch_shinystan(m0b)

##======================================================================
## INLA
library(INLA)

m0i <- inla(y ~ x, data = da)
summary(m0i)
plot(m0i)

m0i$summary.fixed
m0i$summary.hyperpar

str(m0i$internal.marginals.hyperpar)
m <- m0i$internal.marginals.hyperpar[[1]]
m.var <- inla.tmarginal(function(x) 1/exp(x), m)
inla.zmarginal(m.var)
plot(inla.smarginal(m.var), type="l")

str(m0i$internal.marginals.hyperpar)
m <- m0i$internal.marginals.hyperpar[[1]]
m.sd <- inla.tmarginal(function(x) 1/sqrt(exp(x)), m)
inla.zmarginal(m.sd)
plot(inla.smarginal(m.sd), type="l")
